﻿
using System;
using System.Runtime.InteropServices;

namespace ConsoleApp1
{
	unsafe class Program
	{
		#region SDR enums
		private enum mir_sdr_ErrT
		{
			mir_sdr_Success = 0,
			mir_sdr_Fail = 1,
			mir_sdr_InvalidParam = 2,
			mir_sdr_OutOfRange = 3,
			mir_sdr_GainUpdateError = 4,
			mir_sdr_RfUpdateError = 5,
			mir_sdr_FsUpdateError = 6,
			mir_sdr_HwError = 7,
			mir_sdr_AliasingError = 8,
			mir_sdr_AlreadyInitialised = 9,
			mir_sdr_NotInitialised = 10,
			mir_sdr_NotEnabled = 11,
			mir_sdr_HwVerError = 12,
			mir_sdr_OutOfMemError = 13
		}

		private enum mir_sdr_Bw_MHzT
		{
			mir_sdr_BW_Undefined = 0,
			mir_sdr_BW_0_200 = 200,
			mir_sdr_BW_0_300 = 300,
			mir_sdr_BW_0_600 = 600,
			mir_sdr_BW_1_536 = 1536,
			mir_sdr_BW_5_000 = 5000,
			mir_sdr_BW_6_000 = 6000,
			mir_sdr_BW_7_000 = 7000,
			mir_sdr_BW_8_000 = 8000
		}

		private enum mir_sdr_If_kHzT
		{
			mir_sdr_IF_Undefined = -1,
			mir_sdr_IF_Zero = 0,
			mir_sdr_IF_0_450 = 450,
			mir_sdr_IF_1_620 = 1620,
			mir_sdr_IF_2_048 = 2048
		}

		private enum mir_sdr_SetGrModeT
		{
			mir_sdr_USE_SET_GR = 0,
			mir_sdr_USE_SET_GR_ALT_MODE = 1,
			mir_sdr_USE_RSP_SET_GR = 2
		}
		#endregion

		#region SDR DLL imports
		[DllImport("mir_sdr_api.dll", CallingConvention = CallingConvention.Cdecl)]
		static extern mir_sdr_ErrT mir_sdr_StreamUninit();

		[DllImport("mir_sdr_api.dll", CallingConvention = CallingConvention.Cdecl)]
		static extern mir_sdr_ErrT mir_sdr_StreamInit(int *gRdB, double fsMHz, double rfMHz, mir_sdr_Bw_MHzT bwType, mir_sdr_If_kHzT ifType, int LNAstate, 
			          int *gRdBsystem, mir_sdr_SetGrModeT setGrMode, int *samplesPerPacket, mir_sdr_StreamCallback_t StreamCbFn, 
					   mir_sdr_GainChangeCallback_t GainChangeCbFn, void *cbContext);
		#endregion

		#region Callback delegate declarations
		delegate void mir_sdr_StreamCallback_t(short *xi, short *xq, uint firstSampleNum, int grChanged, int rfChanged, int fsChanged, 
			          uint numSamples, uint reset, void *cbContext);

		delegate void mir_sdr_GainChangeCallback_t(uint gRdB, uint lnaGRdB, void *cbContext);
		#endregion

		// callback function pointers
		static mir_sdr_StreamCallback_t streamcbfuncptr;
		static mir_sdr_GainChangeCallback_t gcfuncptr;

		static mir_sdr_ErrT r;

		static int LNAstate = 0;
		static double fsMHz = 10.0;
		static double rfMHz = 95.7;

		static int pktNum;	
		static int testCount;
		static bool running;

		static void Main(string[] args)
		{
			// set callback functions
			streamcbfuncptr = new mir_sdr_StreamCallback_t(streamcbfunc);
			gcfuncptr = new mir_sdr_GainChangeCallback_t(gainChangecbfunc);

			int samplesPerPacket = 0;
			int gRdB = 0;
			int gRdBsystem = 0;
			void* context = null;

			// set test parameters - pktNum is incremented each time the streaming callback fucntion is invoked.
			// test will end when pktNum = testCount
			pktNum = 0;
			testCount = 1000;
			running = true;

			r = mir_sdr_StreamInit(&gRdB, fsMHz, rfMHz, mir_sdr_Bw_MHzT.mir_sdr_BW_1_536, mir_sdr_If_kHzT.mir_sdr_IF_Zero, LNAstate, &gRdBsystem, 
				                   mir_sdr_SetGrModeT.mir_sdr_USE_SET_GR, &samplesPerPacket, streamcbfuncptr, gcfuncptr, context);

			if (r == mir_sdr_ErrT.mir_sdr_Success)
			{
				Console.WriteLine("Streaming started...");
				while (running)
					;
				mir_sdr_StreamUninit();
				Console.WriteLine("Test complete. Press Enter to exit...");
				Console.Read();
			}
			else
				Console.WriteLine("Problem starting SDR streaming");
		}

		static void streamcbfunc(short *xi, short *xq, uint firstSampleNum, int grChanged, int rfChanged, int fsChanged, uint numSamples, uint reset, void *cbContext)
		{
			pktNum++;
			Console.WriteLine("Pkt " + Convert.ToString(pktNum));
			if(pktNum == testCount)
				running = false;
		}

		static void gainChangecbfunc(uint gRdB, uint lnaGRdB, void *cbContext)
		{
		}
	}
}
